@@ -464,7 +464,7 @@ display_all_dupes(struct proc_acl *subject, struct file_acl *filp2)
 	struct stat fstat;
 	struct file_acl ftmp;
 
-	for_each_object(tmp, subject)
+	for_each_object(tmp, subject) {
 	    if (!stat(tmp->filename, &fstat)) {
 		ftmp.inode = fstat.st_ino;
 		if (is_24_kernel)
@@ -472,9 +472,11 @@ display_all_dupes(struct proc_acl *subject, struct file_acl *filp2)
 		else
 			ftmp.dev = MKDEV_26(MAJOR_26(fstat.st_dev), MINOR_26(fstat.st_dev));
 		if (ftmp.inode == filp2->inode && ftmp.dev == filp2->dev)
-			fprintf(stderr, "%s\n", tmp->filename);
+			fprintf(stderr, "%s (due to symlinking/hardlinking)\n", tmp->filename);
+	    } else if (!strcmp(tmp->filename, filp2->filename)) {
+		fprintf(stderr, "%s\n", tmp->filename);
+	    }
 	}
-
 	return;
 }
 
@@ -1035,7 +1037,7 @@ setup_special_roles(struct gr_arg *grarg)
 	for_each_role(rtmp, current_role) {
 		if (rtmp->roletype & GR_ROLE_SPECIAL &&
 		    !(rtmp->roletype & (GR_ROLE_NOPW | GR_ROLE_PAM))) {
-			strncpy(entry.rolename, rtmp->rolename, GR_SPROLE_LEN);
+			strncpy((char *)entry.rolename, rtmp->rolename, GR_SPROLE_LEN);
 			entry.rolename[GR_SPROLE_LEN - 1] = '\0';
 			if (!read_saltandpass
 			    (entry.rolename, entry.salt, entry.sum)) {
@@ -1136,9 +1138,9 @@ conv_user_to_kernel(struct gr_pw_entry *entry)
 	memcpy(&retarg->role_db, role_db, sizeof (struct user_acl_role_db));
       set_pw:
 
-	strncpy(retarg->pw, entry->passwd, GR_PW_LEN - 1);
+	strncpy((char *)retarg->pw, (char *)entry->passwd, GR_PW_LEN - 1);
 	retarg->pw[GR_PW_LEN - 1] = '\0';
-	strncpy(retarg->sp_role, entry->rolename, GR_SPROLE_LEN);
+	strncpy((char *)retarg->sp_role, (char *)entry->rolename, GR_SPROLE_LEN);
 	retarg->sp_role[GR_SPROLE_LEN - 1] = '\0';
 
 	retarg->mode = entry->mode;
